import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter_cocos_view/src/facade_controller.dart';

import 'helpers/misc.dart';

class CocosWidget extends StatefulWidget {
  CocosWidget({
    Key? key,
    required this.onCocosCreated,
    this.onCocosMessage,
    this.fullscreen = false,
    this.enablePlaceholder = false,
    this.runImmediately = false,
    this.unloadOnDispose = false,
    this.printSetupLog = true,
    this.onCocosUnloaded,
    this.gestureRecognizers,
    this.placeholder,
    this.useAndroidViewSurface = false,
    this.onCocosSceneLoaded,
    this.uiLevel = 1,
    this.borderRadius = BorderRadius.zero,
    this.layoutDirection,
    this.hideStatus = false,
  });

  ///Event fires when the Cocos player is created.
  final CocosCreatedCallback onCocosCreated;

  ///Event fires when the [CocosWidget] gets a message from Cocos.
  final CocosMessageCallback? onCocosMessage;

  ///Event fires when the [CocosWidget] gets a scene loaded from Cocos.
  final CocosSceneChangeCallback? onCocosSceneLoaded;

  ///Event fires when the [CocosWidget] Cocos player gets unloaded.
  final CocosUnloadCallback? onCocosUnloaded;

  final Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers;

  /// Set to true to force Cocos to fullscreen
  final bool fullscreen;

  /// Set to true to force Cocos to fullscreen
  final bool hideStatus;

  /// Controls the layer in which Cocos widget is rendered in flutter (defaults to 1)
  final int uiLevel;

  /// This flag tells android to load Cocos as the flutter app starts (Android only)
  final bool runImmediately;

  /// This flag tells android to unload Cocos whenever widget is disposed
  final bool unloadOnDispose;

  /// This flag enables placeholder widget
  final bool enablePlaceholder;

  /// This flag enables placeholder widget
  final bool printSetupLog;

  /// This flag allows you use AndroidView instead of PlatformViewLink for android
  final bool? useAndroidViewSurface;

  /// This is just a helper to render a placeholder widget
  final Widget? placeholder;

  /// Border radius
  final BorderRadius borderRadius;

  /// The layout direction to use for the embedded view.
  ///
  /// If this is null, the ambient [Directionality] is used instead. If there is
  /// no ambient [Directionality], [TextDirection.ltr] is used.
  final TextDirection? layoutDirection;

  @override
  _CocosWidgetState createState() => _CocosWidgetState();
}

class _CocosWidgetState extends State<CocosWidget> {
  @override
  Widget build(BuildContext context) {
    return Text(
        '$defaultTargetPlatform is not yet supported by the Cocos player plugin');
  }
}
