import 'package:flutter_cocos_view/src/helpers/types.dart';

class CocosEvent<T> {
  /// The ID of the Cocos this event is associated to.
  final int cocosId;

  /// The value wrapped by this event
  final T value;

  /// Build a Cocos Event, that relates a mapId with a given value.
  ///
  /// The `cocosId` is the id of the map that triggered the event.
  /// `value` may be `null` in events that don't transport any meaningful data.
  CocosEvent(this.cocosId, this.value);
}

class CocosSceneLoadedEvent extends CocosEvent<SceneLoaded?> {
  CocosSceneLoadedEvent(int cocosId, SceneLoaded? value)
      : super(cocosId, value);
}

class CocosLoadedEvent extends CocosEvent<void> {
  CocosLoadedEvent(int cocosId, void value) : super(cocosId, value);
}

class CocosUnLoadedEvent extends CocosEvent<void> {
  CocosUnLoadedEvent(int cocosId, void value) : super(cocosId, value);
}

class CocosCreatedEvent extends CocosEvent<void> {
  CocosCreatedEvent(int cocosId, void value) : super(cocosId, value);
}

class CocosMessageEvent extends CocosEvent<dynamic> {
  CocosMessageEvent(int cocosId, dynamic value) : super(cocosId, value);
}
