import 'package:flutter_cocos_view/src/helpers/types.dart';

/// Error thrown when an unknown cocos ID is provided to a method channel API.
class UnknownCocosIDError extends Error {
  /// Creates an assertion error with the provided [cocosId] and optional
  /// [message].
  UnknownCocosIDError(this.cocosId, [this.message]);

  /// The unknown ID.
  final int cocosId;

  /// Message describing the assertion error.
  final Object? message;

  String toString() {
    if (message != null) {
      return "Unknown cocos ID $cocosId: ${Error.safeToString(message)}";
    }
    return "Unknown cocos ID $cocosId";
  }
}

typedef void CocosMessageCallback(dynamic handler);

typedef void CocosSceneChangeCallback(SceneLoaded? message);

typedef void CocosUnloadCallback();
