import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import '../helpers/events.dart';
import 'device_method.dart';

abstract class CocosWidgetPlatform extends PlatformInterface {
  /// Constructs a CocosViewFlutterPlatform.
  CocosWidgetPlatform() : super(token: _token);

  static final Object _token = Object();

  static CocosWidgetPlatform _instance = MethodChannelCocosWidget();

  /// The default instance of [CocosWidgetPlatform] to use.
  ///
  /// Defaults to [MethodChannelCocosWidgetFlutter].
  static CocosWidgetPlatform get instance => _instance;

  /// Platform-specific plugins should set this with their own platform-specific
  /// class that extends [CocosWidgetPlatform] when they register themselves.
  static set instance(CocosWidgetPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// /// Initializes the platform interface with [id].
  ///
  /// This method is called when the plugin is first initialized.
  Future<void> init(int cocosId) {
    throw UnimplementedError('init() has not been implemented.');
  }

  Future<bool?> isReady({required int cocosId}) async {
    throw UnimplementedError('init() has not been implemented.');
  }

  Future<bool?> isPaused({required int cocosId}) async {
    throw UnimplementedError('isPaused() has not been implemented.');
  }

  Future<bool?> isLoaded({required int cocosId}) async {
    throw UnimplementedError('isLoaded() has not been implemented.');
  }

  Future<bool?> inBackground({required int cocosId}) async {
    throw UnimplementedError('inBackground() has not been implemented.');
  }

  Future<bool?> createCocosPlayer({required int cocosId}) async {
    throw UnimplementedError('createCocosPlayer() has not been implemented.');
  }

  Future<void> postMessage(
      {required int cocosId,
      required String gameObject,
      required String methodName,
      required String message}) {
    throw UnimplementedError('postMessage() has not been implemented.');
  }

  Future<void> postJsonMessage(
      {required int cocosId,
      required String gameObject,
      required String methodName,
      required Map message}) {
    throw UnimplementedError('postJsonMessage() has not been implemented.');
  }

  Future<void> pausePlayer({required int cocosId}) async {
    throw UnimplementedError('pausePlayer() has not been implemented.');
  }

  Future<void> resumePlayer({required int cocosId}) async {
    throw UnimplementedError('resumePlayer() has not been implemented.');
  }

  /// Opens cocos in it's own activity. Android only.
  Future<void> openInNativeProcess({required int cocosId}) async {
    throw UnimplementedError('openInNativeProcess() has not been implemented.');
  }

  Future<void> unloadPlayer({required int cocosId}) async {
    throw UnimplementedError('unloadPlayer() has not been implemented.');
  }

  Future<void> quitPlayer({required int cocosId}) async {
    throw UnimplementedError('quitPlayer() has not been implemented.');
  }

  Stream<CocosMessageEvent> onCocosMessage({required int cocosId}) {
    throw UnimplementedError('onCocosMessage() has not been implemented.');
  }

  Stream<CocosLoadedEvent> onCocosUnloaded({required int cocosId}) {
    throw UnimplementedError('onCocosUnloaded() has not been implemented.');
  }

  Stream<CocosCreatedEvent> onCocosCreated({required int cocosId}) {
    throw UnimplementedError('onCocosUnloaded() has not been implemented.');
  }

  Stream<CocosSceneLoadedEvent> onCocosSceneLoaded({required int cocosId}) {
    throw UnimplementedError('onCocosSceneLoaded() has not been implemented.');
  }

  /// Dispose of whatever resources the `cocosId` is holding on to.
  void dispose({required int cocosId}) {
    throw UnimplementedError('dispose() has not been implemented.');
  }

  /// Returns a widget displaying the cocos view
  Widget buildView(
    int creationId,
    PlatformViewCreatedCallback onPlatformViewCreated, {
    Map<String, dynamic> cocosOptions = const {},
    Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
    bool? useAndroidViewSurf,
    String? cocosSrcUrl,
  }) {
    throw UnimplementedError('buildView() has not been implemented.');
  }

  /// Returns a widget displaying the cocos view.
  ///
  /// This method is similar to [buildView], but contains a parameter for
  /// platforms that require a text direction.
  ///
  /// Default behavior passes all parameters except `textDirection` to
  /// [buildView]. This is for backward compatibility with existing
  /// implementations. Platforms that use the text direction should override
  /// this as the primary implementation, and delegate to it from buildView.
  Widget buildViewWithTextDirection(
    int creationId,
    PlatformViewCreatedCallback onPlatformViewCreated, {
    required TextDirection textDirection,
    Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers,
    Map<String, dynamic> cocosOptions = const <String, dynamic>{},
    bool? useAndroidViewSurf,
    String? cocosSrcUrl,
  }) {
    return buildView(
      creationId,
      onPlatformViewCreated,
      gestureRecognizers: gestureRecognizers,
      cocosOptions: cocosOptions,
      useAndroidViewSurf: useAndroidViewSurf,
      cocosSrcUrl: cocosSrcUrl,
    );
  }
}
